@@ -13,6 +13,9 @@
 #define HAVE_PRCTL
 #include <mcheck.h>
 #define HAVE_MCHECK
+#define HAVE_MTRACE
+#include <malloc.h>
+#define HAVE_MALLOPT
 #endif
 
 /* Define PY_Type for Python <= 2.6 */
@@ -53,7 +56,6 @@ CPL_list(PyObject *self, PyObject *args) {
     }
 
     cpl_library_t *cpl = create_library(file);
-
     PyObject *res = PyList_New(0);
     Py_INCREF(res);
     cpl_pluginlist *list = cpl->pluginlist_new();
@@ -64,10 +66,12 @@ CPL_list(PyObject *self, PyObject *args) {
 	 plugin = cpl->pluginlist_get_next(list)) {
 	cpl->error_reset();
 	cpl->plugin_get_init(plugin)(plugin);
+	char *version = cpl->plugin_get_version_string(plugin);
 	PyList_Append(res, Py_BuildValue("sis", 
 					 cpl->plugin_get_name(plugin),
 					 cpl->plugin_get_version(plugin),
-					 cpl->plugin_get_version_string(plugin)));
+					 version));
+	cpl->free(version);
 	cpl->plugin_get_deinit(plugin)(plugin);
     }
     cpl->pluginlist_delete(list);
@@ -276,10 +280,17 @@ getParameter(CPL_recipe *self, cpl_parameter *param) {
     }
     Py_INCREF(deflt);
     Py_INCREF(ptype);
+    PyObject *enabled = Py_BuildValue(
+	"OOO",
+	self->cpl->parameter_is_enabled(param, CPL_PARAMETER_MODE_CLI)?Py_True:Py_False,
+	self->cpl->parameter_is_enabled(param, CPL_PARAMETER_MODE_ENV)?Py_True:Py_False,
+	self->cpl->parameter_is_enabled(param, CPL_PARAMETER_MODE_CFG)?Py_True:Py_False);
+    Py_INCREF(enabled);
 
-    PyObject *par = Py_BuildValue("ssssNNNN", 
+    PyObject *par = Py_BuildValue("ssssNNNNN",
 				  name, context, fullname, help,
-				  range, sequence, deflt, ptype);
+				  range, sequence, deflt, ptype,
+				  enabled);
     Py_INCREF(par);
     return par;
 }
@@ -293,7 +304,8 @@ getParameter(CPL_recipe *self, cpl_parameter *param) {
     " - description\n"                                                     \
     " - range (min, max), if valid range is limited, or None\n"            \
     " - allowed values, if only certain values are allowed, or None\n"     \
-    " - default value"
+    " - default value\n"                                                   \
+    " - triple (cli, env, cfg) with enabled-values for param modes"
 
 static PyObject *
 CPL_recipe_get_params(CPL_recipe *self) {
@@ -678,7 +690,7 @@ exec_serialize_retval(CPL_recipe *self, cpl_frameset *frames,
     }
     return ptr;
 }
-static int backtrace(void) {
+static int do_backtrace(void) {
   char cmd[300];
   snprintf(cmd, sizeof(cmd), 
 	   "cat >> gdb_commands << EOF\n"
@@ -701,7 +713,7 @@ static void mcheck_handler(enum mcheck_status status) {
 	   "echo Memory corruption > recipe.backtrace-unprocessed");
   int retval = system(cmd);
   if (retval == 0) {
-      backtrace();
+      do_backtrace();
   }
   abort();
 }
@@ -712,13 +724,13 @@ static int segv_handler(int sig) {
   snprintf(cmd, sizeof(cmd), 
 	   "echo Received signal: %i > recipe.backtrace-unprocessed", sig);
   int retval = system(cmd);
-  backtrace();
+  do_backtrace();
 
   signal(sig, SIG_DFL);
   return retval;
 }
 
-static void setup_tracing(CPL_recipe *self) {
+static void setup_tracing(CPL_recipe *self, int memory_trace) {
 
 #ifdef HAVE_PRCTL
 #ifdef PR_SET_PTRACER
@@ -734,6 +746,15 @@ static void setup_tracing(CPL_recipe *self) {
 #ifdef HAVE_MCHECK
     mcheck(mcheck_handler);
 #endif
+#ifdef HAVE_MALLOPT
+    mallopt(M_CHECK_ACTION, 0);
+#endif
+#ifdef HAVE_MTRACE
+    if (memory_trace) {
+	setenv("MALLOC_TRACE", "recipe.mtrace", 1);
+	mtrace();
+    }
+#endif
 
     typedef void (*sighandler_t)(int);
 
@@ -763,8 +784,10 @@ CPL_recipe_exec(CPL_recipe *self, PyObject *args) {
     const char *logfile;
     int loglevel;
     int memory_dump;
-    if (!PyArg_ParseTuple(args, "sOOOsii", &dirname, &parlist, &soflist,
-			  &runenv, &logfile, &loglevel, &memory_dump))
+    int memory_trace;
+    if (!PyArg_ParseTuple(args, "sOOOsiii", &dirname, &parlist, &soflist,
+			  &runenv, &logfile, &loglevel,
+			  &memory_dump, &memory_trace))
         return NULL;
     if (!PySequence_Check(parlist)) {
 	PyErr_SetString(PyExc_TypeError, "Second parameter not a list");
@@ -816,7 +839,7 @@ CPL_recipe_exec(CPL_recipe *self, PyObject *args) {
 	if (chdir(dirname) == 0) {
 	    struct tms clock_start;
 	    times(&clock_start);
-	    setup_tracing(self);
+	    setup_tracing(self, memory_trace);
 	    retval = self->cpl->plugin_get_exec(self->plugin)(self->plugin);
 	    int reto = self->cpl->dfs_update_product_header(recipe->frames);
 	    if (reto != CPL_ERROR_NONE) {
@@ -832,15 +855,26 @@ CPL_recipe_exec(CPL_recipe *self, PyObject *args) {
 	    retval = CPL_ERROR_FILE_NOT_CREATED;
 	    self->cpl->error_set_message_macro(__func__, retval, __FILE__, __LINE__, " ");
 	}
-	if ((memory_dump > 1) 
-	    || ((memory_dump > 0) && (!self->cpl->memory_is_empty()))) {
-	  self->cpl->memory_dump();
-	}
 	void *ptr = exec_serialize_retval(self, recipe->frames, prestate,
 					  retval, &clock_end);
 	long n_bytes = write(fd[1], ptr, ((long *)ptr)[0]);
 	close(fd[1]);
-	_exit(n_bytes != ((long *)ptr)[0]);
+	retval = (n_bytes != ((long *)ptr)[0]);
+	free(ptr);
+	self->cpl->frameset_delete(recipe->frames);
+	self->cpl->parameterlist_delete(recipe->parameters);
+	recipe->parameters = NULL;
+	recipe->frames = NULL;
+	self->cpl->plugin_get_deinit(self->plugin)(self->plugin);
+	self->cpl->pluginlist_delete(self->pluginlist);
+	Py_TYPE(self)->tp_free((PyObject*)self);
+	if ((memory_dump > 1)
+	    || ((memory_dump > 0) && (!self->cpl->memory_is_empty()))) {
+	  self->cpl->memory_dump();
+	}
+	self->cpl->end();
+	muntrace();
+	_exit(retval);
     }
     
     close(fd[1]);
